/*
 * EtherPix library
 *
 * Copyright 2010-2017 Stefan Schuermans <stefan schuermans info>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <math.h>

#include <etherpix/types.h>
#include "mapping.h"
#include "types.h"

/**
 * \brief precalculate mapping table
 *
 * updates the internal lookup table according to the parameters
 *
 * \param[in,out] p_mapping mapping to precalculate
 */
void etp_mapping_precalc(etp_mapping_t *p_mapping)
{
  double gamma_1;
  int v;

  /* precalculate 1 / gamma */
  gamma_1 = 1.0 / p_mapping->gamma;

  /* map all values */
  for (v = 0; v < 256; v++) {
    double val = p_mapping->base
               + p_mapping->factor * pow((double)v / 255.0, gamma_1);
    if (val < 0.0)
      p_mapping->table[v] = 0;
    else if (val > 1.0)
      p_mapping->table[v] = 255;
    else
      p_mapping->table[v] = (etp_u8_t)(val * 255.0 + 0.5);
  }
}

